@@ -21,37 +21,6 @@ class Test(TestCase):
         f()
         ''', m.UndefinedLocal, m.UnusedVariable)
 
-    @skipIf(version_info >= (3,),
-            'in Python 3 list comprehensions execute in a separate scope')
-    def test_redefinedInListComp(self):
-        """
-        Test that shadowing a variable in a list comprehension raises
-        a warning.
-        """
-        self.flakes('''
-        a = 1
-        [1 for a, b in [(1, 2)]]
-        ''', m.RedefinedInListComp)
-        self.flakes('''
-        class A:
-            a = 1
-            [1 for a, b in [(1, 2)]]
-        ''', m.RedefinedInListComp)
-        self.flakes('''
-        def f():
-            a = 1
-            [1 for a, b in [(1, 2)]]
-        ''', m.RedefinedInListComp)
-        self.flakes('''
-        [1 for a, b in [(1, 2)]]
-        [1 for a, b in [(1, 2)]]
-        ''')
-        self.flakes('''
-        for a, b in [(1, 2)]:
-            pass
-        [1 for a, b in [(1, 2)]]
-        ''')
-
     def test_redefinedInGenerator(self):
         """
         Test that reusing a variable in a generator does not raise
@@ -239,21 +208,6 @@ class Test(TestCase):
             [a for a in '12']
         ''')
 
-    @skipIf(version_info >= (3,),
-            'in Python 3 list comprehensions execute in a separate scope')
-    def test_redefinedElseInListComp(self):
-        """
-        Test that shadowing a variable in a list comprehension in
-        an else (or if) block raises a warning.
-        """
-        self.flakes('''
-        if False:
-            pass
-        else:
-            a = 1
-            [a for a in '12']
-        ''', m.RedefinedInListComp)
-
     def test_functionDecorator(self):
         """
         Test that shadowing a function definition with a decorated version of
@@ -394,7 +348,6 @@ class Test(TestCase):
         yield
         ''', m.YieldOutsideFunction)
 
-    @skipIf(version_info < (3, 3), "Python >= 3.3 only")
     def test_classWithYieldFrom(self):
         """
         If a yield from is used inside a class, a warning is emitted.
@@ -404,7 +357,6 @@ class Test(TestCase):
             yield from range(10)
         ''', m.YieldOutsideFunction)
 
-    @skipIf(version_info < (3, 3), "Python >= 3.3 only")
     def test_moduleWithYieldFrom(self):
         """
         If a yield from is used at the module level, a warning is emitted.
@@ -493,8 +445,10 @@ class Test(TestCase):
                     continue
         ''')
 
+    @skipIf(version_info > (3, 8), "Python <= 3.8 only")
     def test_continueInFinally(self):
         # 'continue' inside 'finally' is a special syntax error
+        # that is removed in 3.8
         self.flakes('''
         while True:
             try:
@@ -901,7 +855,6 @@ class Test(TestCase):
             pass
         ''', m.DefaultExceptNotLast, m.DefaultExceptNotLast)
 
-    @skipIf(version_info < (3,), "Python 3 only")
     def test_starredAssignmentNoError(self):
         """
         Python 3 extended iterable unpacking
@@ -955,7 +908,6 @@ class Test(TestCase):
             ", *rest] = range(1<<8)"
         self.flakes(s)
 
-    @skipIf(version_info < (3, ), "Python 3 only")
     def test_starredAssignmentErrors(self):
         """
         SyntaxErrors (not encoded in the ast) surrounding Python 3 extended
@@ -1159,6 +1111,36 @@ class Test(TestCase):
         def g(): foo = 'anything'; foo.is_used()
         ''')
 
+    def test_function_arguments(self):
+        """
+        Test to traverse ARG and ARGUMENT handler
+        """
+        self.flakes('''
+        def foo(a, b):
+            pass
+        ''')
+
+        self.flakes('''
+        def foo(a, b, c=0):
+            pass
+        ''')
+
+        self.flakes('''
+        def foo(a, b, c=0, *args):
+            pass
+        ''')
+
+        self.flakes('''
+        def foo(a, b, c=0, *args, **kwargs):
+            pass
+        ''')
+
+    def test_function_arguments_python3(self):
+        self.flakes('''
+        def foo(a, b, c=0, *args, d=0, **kwargs):
+            pass
+        ''')
+
 
 class TestUnusedAssignment(TestCase):
     """
@@ -1175,6 +1157,16 @@ class TestUnusedAssignment(TestCase):
             b = 1
         ''', m.UnusedVariable)
 
+    def test_unusedUnderscoreVariable(self):
+        """
+        Don't warn when the magic "_" (underscore) variable is unused.
+        See issue #202.
+        """
+        self.flakes('''
+        def a(unused_param):
+            _ = unused_param
+        ''')
+
     def test_unusedVariableAsLocals(self):
         """
         Using locals() it is perfectly valid to have unused variables
@@ -1233,7 +1225,6 @@ class TestUnusedAssignment(TestCase):
             b = 1
         ''')
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_assignToNonlocal(self):
         """
         Assigning to a nonlocal and then not using that binding is perfectly
@@ -1390,13 +1381,35 @@ class TestUnusedAssignment(TestCase):
         self.flakes("a = foo if True else 'oink'", m.UndefinedName)
         self.flakes("a = 'moo' if True else bar", m.UndefinedName)
 
+    def test_if_tuple(self):
+        """
+        Test C{if (foo,)} conditions.
+        """
+        self.flakes("""if (): pass""")
+        self.flakes("""
+        if (
+            True
+        ):
+            pass
+        """)
+        self.flakes("""
+        if (
+            True,
+        ):
+            pass
+        """, m.IfTuple)
+        self.flakes("""
+        x = 1 if (
+            True,
+        ) else 2
+        """, m.IfTuple)
+
     def test_withStatementNoNames(self):
         """
         No warnings are emitted for using inside or after a nameless C{with}
         statement a name defined beforehand.
         """
         self.flakes('''
-        from __future__ import with_statement
         bar = None
         with open("foo"):
             bar
@@ -1409,7 +1422,6 @@ class TestUnusedAssignment(TestCase):
         within the suite or afterwards.
         """
         self.flakes('''
-        from __future__ import with_statement
         with open('foo') as bar:
             bar
         bar
@@ -1421,7 +1433,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         import foo
         with open('foo') as foo.bar:
             pass
@@ -1433,7 +1444,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         import foo
         with open('foo') as foo[0]:
             pass
@@ -1445,7 +1455,6 @@ class TestUnusedAssignment(TestCase):
         target of a C{with} statement is not defined.
         """
         self.flakes('''
-        from __future__ import with_statement
         import foo
         with open('foo') as foo[bar]:
             pass
@@ -1457,7 +1466,6 @@ class TestUnusedAssignment(TestCase):
         a C{with} statement within the suite or afterwards.
         """
         self.flakes('''
-        from __future__ import with_statement
         with open('foo') as (bar, baz):
             bar, baz
         bar, baz
@@ -1469,7 +1477,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement within the suite or afterwards.
         """
         self.flakes('''
-        from __future__ import with_statement
         with open('foo') as [bar, baz]:
             bar, baz
         bar, baz
@@ -1484,7 +1491,6 @@ class TestUnusedAssignment(TestCase):
         created are respected in the suite of the statement and afterwards.
         """
         self.flakes('''
-        from __future__ import with_statement
         c = d = e = g = h = i = None
         with open('foo') as [(a, b), c[d], e.f, g[h:i]]:
             a, b, c, d, e, g, h, i
@@ -1497,7 +1503,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement is used before the C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         bar
         with open('foo') as bar:
             pass
@@ -1510,7 +1515,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         baz
         with open('foo') as (bar, baz):
             pass
@@ -1522,7 +1526,6 @@ class TestUnusedAssignment(TestCase):
         rebound by the name defined by a C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         import bar
         with open('foo') as bar:
             pass
@@ -1535,7 +1538,6 @@ class TestUnusedAssignment(TestCase):
         C{with} statement.
         """
         self.flakes('''
-        from __future__ import with_statement
         import bar
         with open('foo') as (bar, baz):
             pass
@@ -1547,7 +1549,6 @@ class TestUnusedAssignment(TestCase):
         body of a C{with} statement without first being bound.
         """
         self.flakes('''
-        from __future__ import with_statement
         with open('foo') as bar:
             baz
         ''', m.UndefinedName)
@@ -1558,7 +1559,6 @@ class TestUnusedAssignment(TestCase):
         the body ends without warning.
         """
         self.flakes('''
-        from __future__ import with_statement
         with open('foo') as bar:
             baz = 10
         baz
@@ -1570,13 +1570,11 @@ class TestUnusedAssignment(TestCase):
         expression of a C{with} statement is undefined.
         """
         self.flakes('''
-        from __future__ import with_statement
         with bar as baz:
             pass
         ''', m.UndefinedName)
 
         self.flakes('''
-        from __future__ import with_statement
         with bar as bar:
             pass
         ''', m.UndefinedName)
@@ -1610,14 +1608,21 @@ class TestUnusedAssignment(TestCase):
             except Exception as e: e
         ''')
 
-    @skipIf(version_info < (3,),
-            "In Python 2 exception names stay bound after the exception handler")
     def test_exceptionUnusedInExcept(self):
         self.flakes('''
         try: pass
         except Exception as e: pass
         ''', m.UnusedVariable)
 
+    @skipIf(version_info < (3, 11), 'new in Python 3.11')
+    def test_exception_unused_in_except_star(self):
+        self.flakes('''
+            try:
+                pass
+            except* OSError as e:
+                pass
+        ''', m.UnusedVariable)
+
     def test_exceptionUnusedInExceptInFunction(self):
         self.flakes('''
         def download_review():
@@ -1695,7 +1700,6 @@ class TestUnusedAssignment(TestCase):
         assert 1
         ''')
 
-    @skipIf(version_info < (3, 3), 'new in Python 3.3')
     def test_yieldFromUndefined(self):
         """
         Test C{yield from} statement
@@ -1705,7 +1709,6 @@ class TestUnusedAssignment(TestCase):
             yield from foo()
         ''', m.UndefinedName)
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_f_string(self):
         """Test PEP 498 f-strings are treated as a usage."""
         self.flakes('''
@@ -1713,31 +1716,204 @@ class TestUnusedAssignment(TestCase):
         print(f'\x7b4*baz\N{RIGHT CURLY BRACKET}')
         ''')
 
+    @skipIf(version_info < (3, 8), 'new in Python 3.8')
+    def test_assign_expr(self):
+        """Test PEP 572 assignment expressions are treated as usage / write."""
+        self.flakes('''
+        from foo import y
+        print(x := y)
+        print(x)
+        ''')
+
+    @skipIf(version_info < (3, 8), 'new in Python 3.8')
+    def test_assign_expr_generator_scope(self):
+        """Test assignment expressions in generator expressions."""
+        self.flakes('''
+        if (any((y := x[0]) for x in [[True]])):
+            print(y)
+        ''')
+
+    @skipIf(version_info < (3, 8), 'new in Python 3.8')
+    def test_assign_expr_nested(self):
+        """Test assignment expressions in nested expressions."""
+        self.flakes('''
+        if ([(y:=x) for x in range(4) if [(z:=q) for q in range(4)]]):
+            print(y)
+            print(z)
+        ''')
+
+
+class TestStringFormatting(TestCase):
+
+    def test_f_string_without_placeholders(self):
+        self.flakes("f'foo'", m.FStringMissingPlaceholders)
+        self.flakes('''
+            f"""foo
+            bar
+            """
+        ''', m.FStringMissingPlaceholders)
+        self.flakes('''
+            print(
+                f'foo'
+                f'bar'
+            )
+        ''', m.FStringMissingPlaceholders)
+        # this is an "escaped placeholder" but not a placeholder
+        self.flakes("f'{{}}'", m.FStringMissingPlaceholders)
+        # ok: f-string with placeholders
+        self.flakes('''
+            x = 5
+            print(f'{x}')
+        ''')
+        # ok: f-string with format specifiers
+        self.flakes('''
+            x = 'a' * 90
+            print(f'{x:.8}')
+        ''')
+        # ok: f-string with multiple format specifiers
+        self.flakes('''
+            x = y = 5
+            print(f'{x:>2} {y:>2}')
+        ''')
+
+    def test_invalid_dot_format_calls(self):
+        self.flakes('''
+            '{'.format(1)
+        ''', m.StringDotFormatInvalidFormat)
+        self.flakes('''
+            '{} {1}'.format(1, 2)
+        ''', m.StringDotFormatMixingAutomatic)
+        self.flakes('''
+            '{0} {}'.format(1, 2)
+        ''', m.StringDotFormatMixingAutomatic)
+        self.flakes('''
+            '{}'.format(1, 2)
+        ''', m.StringDotFormatExtraPositionalArguments)
+        self.flakes('''
+            '{}'.format(1, bar=2)
+        ''', m.StringDotFormatExtraNamedArguments)
+        self.flakes('''
+            '{} {}'.format(1)
+        ''', m.StringDotFormatMissingArgument)
+        self.flakes('''
+            '{2}'.format()
+        ''', m.StringDotFormatMissingArgument)
+        self.flakes('''
+            '{bar}'.format()
+        ''', m.StringDotFormatMissingArgument)
+        # too much string recursion (placeholder-in-placeholder)
+        self.flakes('''
+            '{:{:{}}}'.format(1, 2, 3)
+        ''', m.StringDotFormatInvalidFormat)
+        # ok: dotted / bracketed names need to handle the param differently
+        self.flakes("'{.__class__}'.format('')")
+        self.flakes("'{foo[bar]}'.format(foo={'bar': 'barv'})")
+        # ok: placeholder-placeholders
+        self.flakes('''
+            print('{:{}} {}'.format(1, 15, 2))
+        ''')
+        # ok: not a placeholder-placeholder
+        self.flakes('''
+            print('{:2}'.format(1))
+        ''')
+        # ok: not mixed automatic
+        self.flakes('''
+            '{foo}-{}'.format(1, foo=2)
+        ''')
+        # ok: we can't determine statically the format args
+        self.flakes('''
+            a = ()
+            "{}".format(*a)
+        ''')
+        self.flakes('''
+            k = {}
+            "{foo}".format(**k)
+        ''')
+
+    def test_invalid_percent_format_calls(self):
+        self.flakes('''
+            '%(foo)' % {'foo': 'bar'}
+        ''', m.PercentFormatInvalidFormat)
+        self.flakes('''
+            '%s %(foo)s' % {'foo': 'bar'}
+        ''', m.PercentFormatMixedPositionalAndNamed)
+        self.flakes('''
+            '%(foo)s %s' % {'foo': 'bar'}
+        ''', m.PercentFormatMixedPositionalAndNamed)
+        self.flakes('''
+            '%j' % (1,)
+        ''', m.PercentFormatUnsupportedFormatCharacter)
+        self.flakes('''
+            '%s %s' % (1,)
+        ''', m.PercentFormatPositionalCountMismatch)
+        self.flakes('''
+            '%s %s' % (1, 2, 3)
+        ''', m.PercentFormatPositionalCountMismatch)
+        self.flakes('''
+            '%(bar)s' % {}
+        ''', m.PercentFormatMissingArgument,)
+        self.flakes('''
+            '%(bar)s' % {'bar': 1, 'baz': 2}
+        ''', m.PercentFormatExtraNamedArguments)
+        self.flakes('''
+            '%(bar)s' % (1, 2, 3)
+        ''', m.PercentFormatExpectedMapping)
+        self.flakes('''
+            '%s %s' % {'k': 'v'}
+        ''', m.PercentFormatExpectedSequence)
+        self.flakes('''
+            '%(bar)*s' % {'bar': 'baz'}
+        ''', m.PercentFormatStarRequiresSequence)
+        # ok: single %s with mapping
+        self.flakes('''
+            '%s' % {'foo': 'bar', 'baz': 'womp'}
+        ''')
+        # ok: does not cause a MemoryError (the strings aren't evaluated)
+        self.flakes('''
+            "%1000000000000f" % 1
+        ''')
+        # ok: %% should not count towards placeholder count
+        self.flakes('''
+            '%% %s %% %s' % (1, 2)
+        ''')
+        # ok: * consumes one positional argument
+        self.flakes('''
+            '%.*f' % (2, 1.1234)
+            '%*.*f' % (5, 2, 3.1234)
+        ''')
+
+    def test_ok_percent_format_cannot_determine_element_count(self):
+        self.flakes('''
+            a = []
+            '%s %s' % [*a]
+            '%s %s' % (*a,)
+        ''')
+        self.flakes('''
+            k = {}
+            '%(k)s' % {**k}
+        ''')
+
 
 class TestAsyncStatements(TestCase):
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncDef(self):
         self.flakes('''
         async def bar():
             return 42
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncDefAwait(self):
         self.flakes('''
         async def read_data(db):
             await db.fetch('SELECT ...')
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncDefUndefined(self):
         self.flakes('''
         async def bar():
             return foo()
         ''', m.UndefinedName)
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncFor(self):
         self.flakes('''
         async def read_data(db):
@@ -1747,7 +1923,13 @@ class TestAsyncStatements(TestCase):
             return output
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
+    def test_asyncForUnderscoreLoopVar(self):
+        self.flakes('''
+        async def coro(it):
+            async for _ in it:
+                pass
+        ''')
+
     def test_loopControlInAsyncFor(self):
         self.flakes('''
         async def read_data(db):
@@ -1769,7 +1951,6 @@ class TestAsyncStatements(TestCase):
             return output
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_loopControlInAsyncForElse(self):
         self.flakes('''
         async def read_data(db):
@@ -1791,7 +1972,7 @@ class TestAsyncStatements(TestCase):
             return output
         ''', m.BreakOutsideLoop)
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
+    @skipIf(version_info > (3, 8), "Python <= 3.8 only")
     def test_continueInAsyncForFinally(self):
         self.flakes('''
         async def read_data(db):
@@ -1804,7 +1985,6 @@ class TestAsyncStatements(TestCase):
             return output
         ''', m.ContinueInFinally)
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncWith(self):
         self.flakes('''
         async def commit(session, data):
@@ -1812,7 +1992,6 @@ class TestAsyncStatements(TestCase):
                 await session.update(data)
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_asyncWithItem(self):
         self.flakes('''
         async def commit(session, data):
@@ -1822,14 +2001,12 @@ class TestAsyncStatements(TestCase):
                 await trans.end()
         ''')
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_matmul(self):
         self.flakes('''
         def foo(a, b):
             return a @ b
         ''')
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_formatstring(self):
         self.flakes('''
         hi = 'hi'
@@ -1837,146 +2014,6 @@ class TestAsyncStatements(TestCase):
         f'{hi} {mom}'
         ''')
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
-    def test_variable_annotations(self):
-        self.flakes('''
-        name: str
-        age: int
-        ''')
-        self.flakes('''
-        name: str = 'Bob'
-        age: int = 18
-        ''')
-        self.flakes('''
-        class C:
-            name: str
-            age: int
-        ''')
-        self.flakes('''
-        class C:
-            name: str = 'Bob'
-            age: int = 18
-        ''')
-        self.flakes('''
-        def f():
-            name: str
-            age: int
-        ''')
-        self.flakes('''
-        def f():
-            name: str = 'Bob'
-            age: int = 18
-            foo: not_a_real_type = None
-        ''', m.UnusedVariable, m.UnusedVariable, m.UnusedVariable, m.UndefinedName)
-        self.flakes('''
-        def f():
-            name: str
-            print(name)
-        ''', m.UndefinedName)
-        self.flakes('''
-        from typing import Any
-        def f():
-            a: Any
-        ''')
-        self.flakes('''
-        foo: not_a_real_type
-        ''', m.UndefinedName)
-        self.flakes('''
-        foo: not_a_real_type = None
-        ''', m.UndefinedName)
-        self.flakes('''
-        class C:
-            foo: not_a_real_type
-        ''', m.UndefinedName)
-        self.flakes('''
-        class C:
-            foo: not_a_real_type = None
-        ''', m.UndefinedName)
-        self.flakes('''
-        def f():
-            class C:
-                foo: not_a_real_type
-        ''', m.UndefinedName)
-        self.flakes('''
-        def f():
-            class C:
-                foo: not_a_real_type = None
-        ''', m.UndefinedName)
-        self.flakes('''
-        from foo import Bar
-        bar: Bar
-        ''')
-        self.flakes('''
-        from foo import Bar
-        bar: 'Bar'
-        ''')
-        self.flakes('''
-        import foo
-        bar: foo.Bar
-        ''')
-        self.flakes('''
-        import foo
-        bar: 'foo.Bar'
-        ''')
-        self.flakes('''
-        from foo import Bar
-        def f(bar: Bar): pass
-        ''')
-        self.flakes('''
-        from foo import Bar
-        def f(bar: 'Bar'): pass
-        ''')
-        self.flakes('''
-        from foo import Bar
-        def f(bar) -> Bar: return bar
-        ''')
-        self.flakes('''
-        from foo import Bar
-        def f(bar) -> 'Bar': return bar
-        ''')
-        self.flakes('''
-        bar: 'Bar'
-        ''', m.UndefinedName)
-        self.flakes('''
-        bar: 'foo.Bar'
-        ''', m.UndefinedName)
-        self.flakes('''
-        from foo import Bar
-        bar: str
-        ''', m.UnusedImport)
-        self.flakes('''
-        from foo import Bar
-        def f(bar: str): pass
-        ''', m.UnusedImport)
-        self.flakes('''
-        def f(a: A) -> A: pass
-        class A: pass
-        ''', m.UndefinedName, m.UndefinedName)
-        self.flakes('''
-        def f(a: 'A') -> 'A': return a
-        class A: pass
-        ''')
-        self.flakes('''
-        a: A
-        class A: pass
-        ''', m.UndefinedName)
-        self.flakes('''
-        a: 'A'
-        class A: pass
-        ''')
-        self.flakes('''
-        a: 'A B'
-        ''', m.ForwardAnnotationSyntaxError)
-        self.flakes('''
-        a: 'A; B'
-        ''', m.ForwardAnnotationSyntaxError)
-        self.flakes('''
-        a: '1 + 2'
-        ''')
-        self.flakes('''
-        a: 'a: "A"'
-        ''', m.ForwardAnnotationSyntaxError)
-
     def test_raise_notimplemented(self):
         self.flakes('''
         raise NotImplementedError("This is fine")
@@ -1993,3 +2030,57 @@ class TestAsyncStatements(TestCase):
         self.flakes('''
         raise NotImplemented
         ''', m.RaiseNotImplemented)
+
+
+class TestIncompatiblePrintOperator(TestCase):
+    """
+    Tests for warning about invalid use of print function.
+    """
+
+    def test_valid_print(self):
+        self.flakes('''
+        print("Hello")
+        ''')
+
+    def test_invalid_print_when_imported_from_future(self):
+        exc = self.flakes('''
+        from __future__ import print_function
+        import sys
+        print >>sys.stderr, "Hello"
+        ''', m.InvalidPrintSyntax).messages[0]
+
+        self.assertEqual(exc.lineno, 4)
+        self.assertEqual(exc.col, 0)
+
+    def test_print_augmented_assign(self):
+        # nonsense, but shouldn't crash pyflakes
+        self.flakes('print += 1')
+
+    def test_print_function_assignment(self):
+        """
+        A valid assignment, tested for catching false positives.
+        """
+        self.flakes('''
+        from __future__ import print_function
+        log = print
+        log("Hello")
+        ''')
+
+    def test_print_in_lambda(self):
+        self.flakes('''
+        from __future__ import print_function
+        a = lambda: print
+        ''')
+
+    def test_print_returned_in_function(self):
+        self.flakes('''
+        from __future__ import print_function
+        def a():
+            return print
+        ''')
+
+    def test_print_as_condition_test(self):
+        self.flakes('''
+        from __future__ import print_function
+        if print: pass
+        ''')
